package ${basePackage}.module.system.mgr;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.frame.auth.Token;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.utils.IDgenerator;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.utils.Message;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.Sequence;
import ${basePackage}.module.system.mpr.SequenceMapper;
import ${basePackage}.module.system.req.SequenceCreateRequest;
import ${basePackage}.module.system.req.SequenceDeleteRequest;
import ${basePackage}.module.system.req.SequenceFindRequest;
import ${basePackage}.module.system.req.SequenceGetRequest;
import ${basePackage}.module.system.req.SequenceUpdateRequest;
import ${basePackage}.module.system.rsp.SequenceCreateResponse;
import ${basePackage}.module.system.rsp.SequenceDeleteResponse;
import ${basePackage}.module.system.rsp.SequenceFindResponse;
import ${basePackage}.module.system.rsp.SequenceGetResponse;
import ${basePackage}.module.system.rsp.SequenceUpdateResponse;

/**
 * SEQUENCE - 序列
 *
 * @author wangbing
 * @version 0.0.1
 * @since 2020-06-25
 */
@Transactional
@Service
public class SequenceManagerImpl implements ${basePackage}.module.system.mgr.SequenceManager {

    @Autowired
    private SequenceMapper sequenceMapper;

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public SequenceCreateResponse create(SequenceCreateRequest request, Token token) {
        SequenceCreateResponse response = new SequenceCreateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long id = IDgenerator.nextId();
        Sequence entity = MapperUtil.map(request, Sequence.class);
        entity.setId(id);

        long result = sequenceMapper.insert(entity, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.CREATE_FAILURE);
            return response;
        }
        response.setId(id);

        return response;
    }

    /**
     * 逻辑删除
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public SequenceDeleteResponse delete(SequenceDeleteRequest request, Token token) {
        SequenceDeleteResponse response = new SequenceDeleteResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = sequenceMapper.delete(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.DELETE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public SequenceUpdateResponse update(SequenceUpdateRequest request, Token token) {
        SequenceUpdateResponse response = new SequenceUpdateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = sequenceMapper.update(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.UPDATE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public SequenceFindResponse find(SequenceFindRequest request, Token token) {
        SequenceFindResponse response = new SequenceFindResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        if (request.getPageSize() != 0) {
            PageHelper.startPage(request.getPageNumber(), request.getPageSize());
        }
        if (StringUtil.isNotEmpty(request.getSortKey())) {
            PageHelper.orderBy(request.getSortKey() + " " + request.getSortType());
        }
        PageInfo<Sequence> pageInfo = new PageInfo<>(sequenceMapper.find(request, token));

        response.setResult(pageInfo.getList());
        response.setTotalCount(pageInfo.getTotal());

        return response;
    }

    /**
     * 获得对象
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public SequenceGetResponse get(SequenceGetRequest request, Token token) {
        SequenceGetResponse response = new SequenceGetResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        Sequence po = sequenceMapper.get(request, token);

        if (po != null) {
            response.setSequence(po);
        } else {
            response.addError(ErrorType.BUSINESS_ERROR, Message.GET_FAILURE);
        }

        return response;
    }
}
